# Copyright 2021 Gergö Gabor Ilyes-Veisz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

ARGUMENT_FRAME_SENDER = "frame_sender"
ARGUMENT_HEARTBEAT = "heartbeat"
ARGUMENT_LOGGER = "logger"

CONNECTION_OPTION_KEY_IP_ADDRESS = "ip_address"
CONNECTION_OPTION_KEY_API_SECRET_KEY = "api_secret_key"
CONNECTION_OPTION_KEY_API_AUTH_KEY = "api_auth_key"
DEVICE_CONNECTING_MAXIMUM_ATTEMPTS = 10
DEVICE_CONNECTING_HEARTBEAT = 2.5
MESSAGE_HANDLER_HEARTBEAT = 10
PING_SENDER_HEARTBEAT = 60
AUTHENTICATOR_HEARTBEAT = MESSAGE_HANDLER_HEARTBEAT
ENCODING = "latin-1"



# Copyright 2021 Gergö Gabor Ilyes-Veisz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum


class State(Enum):
    """
    Enumeration of possible states of the client (``aioremootio.RemootioClient``).
    """

    CLOSED = "closed"
    OPEN = "open"
    CLOSING = "closing"
    OPENING = "opening"
    NO_SENSOR_INSTALLED = "no sensor"
    UNKNOWN = "unknown"

    @classmethod
    def _missing_(cls, value):
        return State.UNKNOWN


class ErrorType(Enum):
    JSON_ERROR = "json error"
    INPUT_ERROR = "input error"
    INTERNAL_ERROR = "internal error"
    CONNECTION_TIMEOUT = "connection timeout"
    AUTHENTICATION_TIMEOUT = "authentication timeout"
    AUTHENTICATION_ERROR = "authentication error"
    ALREADY_AUTHENTICATED = "already authenticated"
    UNKNOWN = "unknown error"

    @classmethod
    def _missing_(cls, value):
        return ErrorType.UNKNOWN


class ActionType(Enum):
    QUERY = "QUERY"
    TRIGGER = "TRIGGER"
    OPEN = "OPEN"
    CLOSE = "CLOSE"
    UNSUPPORTED = "UNSUPPORTED"

    @classmethod
    def _missing_(cls, value):
        return ActionType.UNSUPPORTED


class EventType(Enum):
    """
    Enumeration of possible kinds of event. For more information see ``aioremootio.models.Event``.
    """

    STATE_CHANGE = "StateChange"
    RELAY_TRIGGER = "RelayTrigger"
    LEFT_OPEN = "LeftOpen"
    UNSUPPORTED = "Unsupported"

    @classmethod
    def _missing_(cls, value):
        return EventType.UNSUPPORTED


class KeyType(Enum):
    """
    Enumeration of possible kinds of keys. For more information see ``aioremootio.models.Key``.
    """

    MASTER_KEY = "master key"
    UNIQUE_KEY = "unique key"
    GUEST_KEY = "guest key"
    API_KEY = "api key"
    SMART_HOME = "smart home"
    AUTOMATION = "automation"
    UNSUPPORTED = "Unsupported"

    @classmethod
    def _missing_(cls, value):
        return KeyType.UNSUPPORTED


class DeviceType(Enum):
    REMOOTIO_1 = "remootio-1"
    REMOOTIO_2 = "remootio-2"
    UNKNOWN = "unknown"

    @classmethod
    def _missing_(cls, value):
        return DeviceType.UNKNOWN


class FrameType(Enum):
    ENCRYPTED = "ENCRYPTED"
    SERVER_HELLO = "SERVER_HELLO"
    PONG = "PONG"
    ERROR = "ERROR"
    CHALLENGE = "CHALLENGE"


class ErrorCode(Enum):
    RELAY_BUSY = "ERR_RELAY_BUSY"
    INVALID_REQUEST = "ERR_INVALID_REQUEST"
    NO_SENSOR = "ERR_NO_SENSOR"
    UNKNOWN = "unknown"

    @classmethod
    def _missing_(cls, value):
        return ErrorCode.UNKNOWN


class EventSource(Enum):
    """
    Enumeration of possible connection methods which can be used to trigger an event. For more information see
    ``aioremootio.models.Event``.
    """

    BLUETOOTH = "bluetooth"
    WIFI = "wifi"
    INTERNET = "internet"
    AUTOOPEN = "autoopen"
    UNKNOWN = "unknown"
    NONE = "none"

    @classmethod
    def _missing_(cls, value):
        return EventSource.UNKNOWN



# Copyright 2021 Gergö Gabor Ilyes-Veisz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NoReturn, TypeVar, Generic, TYPE_CHECKING
from abc import ABC, abstractmethod
if TYPE_CHECKING:
    from .remootioclient import RemootioClient

T = TypeVar('T')


class Listener(ABC, Generic[T]):
    """
    Abstract class for listener implementations which will be invoked by the client (``aioremootio.RemootioClient``).
    """

    @abstractmethod
    async def execute(self, client: 'RemootioClient', subject: T) -> NoReturn:
        """
        Method called by the client.
        :param client: the ``aioremootio.RemootioClient`` which has invoked this listener
        :param subject: the subject because of the ``aioremootio.RemootioClient`` has invoked the listener
        """
        pass



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')
install_requires = list(val.strip() for val in open(here / 'requirements.txt'))

setup(
    name="aioremootio",
    version="1.0.0.alpha3",
    description="An asynchronous API client library for Remootio (http://www.remootio.com/)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ivgg-me/aioremootio",
    author="Gergö Gabor Ilyes-Veisz",
    author_email="i@ivgg.me",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Framework :: AsyncIO",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.9",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries"
    ],
    keywords="remootio, client, library, asynchronous",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.9",
    install_requires=install_requires,
    project_urls={
        "Bug Tracker": "https://github.com/ivgg-me/aioremootio/issues",
        "Source": "https://github.com/ivgg-me/aioremootio/"
    }
)


# Copyright 2021 Gergö Gabor Ilyes-Veisz
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NoReturn, TYPE_CHECKING
if TYPE_CHECKING:
    from .remootioclient import RemootioClient


class RemootioError(Exception):
    """Base class for aioremootio errors."""

    def __init__(self, device: 'RemootioClient', message: str) -> NoReturn:
        super().__init__(message)
        self.__message = message
        self.__device = device

    @property
    def message(self):
        return self.__message

    @property
    def remootio_device(self) -> 'RemootioClient':
        return self.__device


class RemootioClientError(RemootioError):

    def __init__(self, device: 'RemootioClient', message: str) -> NoReturn:
        super().__init__(device, message)


class RemootioClientDecryptionError(RemootioClientError):

    def __init__(self, device: 'RemootioClient', message: str) -> NoReturn:
        super().__init__(device, message)


class RemootioClientEncryptionError(RemootioClientError):

    def __init__(self, device: 'RemootioClient', message: str) -> NoReturn:
        super().__init__(device, message)


class RemootioClientUnsupportedOperationError(RemootioError):

    def __init__(self, device: 'RemootioClient', message: str) -> NoReturn:
        super().__init__(device, message)
